@@ -132,6 +132,8 @@ apr_byte_t oidc_cache_mutex_post_config(server_rec *s, oidc_cache_mutex_t *m,
 	}
 #endif
 
+	apr_global_mutex_lock(m->mutex);
+
 	rv = apr_shm_create(&m->shm, sizeof(int), NULL, s->process->pool);
 	if (rv != APR_SUCCESS) {
 		oidc_serror(s, "apr_shm_create failed to create shared memory segment");
@@ -141,6 +143,8 @@ apr_byte_t oidc_cache_mutex_post_config(server_rec *s, oidc_cache_mutex_t *m,
 	m->sema = apr_shm_baseaddr_get(m->shm);
 	*m->sema = 1;
 
+	apr_global_mutex_unlock(m->mutex);
+
 	return TRUE;
 }
 
@@ -210,19 +214,25 @@ apr_byte_t oidc_cache_mutex_destroy(server_rec *s, oidc_cache_mutex_t *m) {
 		apr_global_mutex_lock(m->mutex);
 		(*m->sema)--;
 		//oidc_sdebug(s, "semaphore: %d (m=%pp,s=%pp)", *m->sema, m->mutex, s);
-		apr_global_mutex_unlock(m->mutex);
 
 		if ((m->shm != NULL) && (*m->sema == 0)) {
 
-			rv = apr_global_mutex_destroy(m->mutex);
-			oidc_sdebug(s, "apr_global_mutex_destroy returned :%d", rv);
-			m->mutex = NULL;
-
 			rv = apr_shm_destroy(m->shm);
 			oidc_sdebug(s, "apr_shm_destroy for semaphore returned: %d", rv);
 			m->shm = NULL;
 
+			apr_global_mutex_unlock(m->mutex);
+
+			rv = apr_global_mutex_destroy(m->mutex);
+			oidc_sdebug(s, "apr_global_mutex_destroy returned :%d", rv);
+			m->mutex = NULL;
+
 			rv = APR_SUCCESS;
+
+		} else {
+
+			apr_global_mutex_unlock(m->mutex);
+
 		}
 	}
 
