@@ -107,7 +107,7 @@ static apr_byte_t oidc_oauth_provider_config(request_rec *r, oidc_cfg *c) {
 			return FALSE;
 		}
 
-		oidc_cache_set_provider(r, c->oauth.metadata_url, s_json,
+		oidc_cache_set_oauth_provider(r, c->oauth.metadata_url, s_json,
 				apr_time_now() + (c->provider_metadata_refresh_interval <= 0 ? apr_time_from_sec( OIDC_CACHE_PROVIDER_METADATA_EXPIRY_DEFAULT) : c->provider_metadata_refresh_interval));
 
 	} else {
@@ -168,8 +168,9 @@ static apr_byte_t oidc_oauth_validate_access_token(request_rec *r, oidc_cfg *c,
 	/* add the token endpoint authentication credentials */
 	if (oidc_proto_token_endpoint_auth(r, c,
 			c->oauth.introspection_endpoint_auth, c->oauth.client_id,
-			c->oauth.client_secret, c->oauth.introspection_endpoint_url, params,
-			bearer_access_token_auth, &basic_auth, &bearer_auth) == FALSE)
+			c->oauth.client_secret, NULL, c->oauth.introspection_endpoint_url,
+			params, bearer_access_token_auth, &basic_auth,
+			&bearer_auth) == FALSE)
 		return FALSE;
 
 	/* call the endpoint with the constructed parameter set and return the resulting response */
@@ -263,7 +264,8 @@ apr_byte_t oidc_oauth_get_bearer_token(request_rec *r,
 	if ((*access_token == NULL) && (r->method_number == M_POST)
 			&& (accept_token_in & OIDC_OAUTH_ACCEPT_TOKEN_IN_POST)) {
 		apr_table_t *params = apr_table_make(r->pool, 8);
-		if (oidc_util_read_post_params(r, params) == TRUE) {
+		if (oidc_util_read_post_params(r, params, TRUE,
+				OIDC_PROTO_ACCESS_TOKEN) == TRUE) {
 			*access_token = apr_table_get(params, OIDC_PROTO_ACCESS_TOKEN);
 		}
 	}
@@ -640,7 +642,7 @@ static apr_byte_t oidc_oauth_validate_jwt_access_token(request_rec *r,
 			c->provider.jwks_refresh_interval, c->oauth.ssl_validate_server };
 	if (oidc_proto_jwt_verify(r, c, jwt, &jwks_uri,
 			oidc_util_merge_key_sets(r->pool, c->oauth.verify_public_keys,
-					c->oauth.verify_shared_keys)) == FALSE) {
+					c->oauth.verify_shared_keys), NULL) == FALSE) {
 		oidc_error(r,
 				"JWT access token signature could not be validated, aborting");
 		oidc_jwt_destroy(jwt);
@@ -737,16 +739,22 @@ int oidc_oauth_check_userid(request_rec *r, oidc_cfg *c,
 			return OK;
 		}
 
-		/* check if this is a request for the public (encryption) keys */
+		/* check if this is a request to the "special" handler (Redirect URI) */
 	} else if (oidc_util_request_matches_url(r, oidc_get_redirect_uri(r, c))) {
 
+		/* check if this is a request for the public (encryption) keys */
 		if (oidc_util_request_has_parameter(r,
 				OIDC_REDIRECT_URI_REQUEST_JWKS)) {
 
 			return oidc_handle_jwks(r, c);
 
-		}
+			/* check if this is a request to remove the access token from the cache */
+		} else if (oidc_util_request_has_parameter(r,
+				OIDC_REDIRECT_URI_REQUEST_REMOVE_AT_CACHE)) {
 
+			/* handle request to invalidate access token cache */
+			return oidc_handle_remove_at_cache(r, c);
+		}
 	}
 
 	/* we don't have a session yet */
