@@ -274,7 +274,9 @@ const char *oidc_parse_cache_type(apr_pool_t *pool, const char *arg,
 		oidc_cache_t **type) {
 	static char *options[] = {
 			OIDC_CACHE_TYPE_SHM,
+#ifdef USE_MEMCACHE
 			OIDC_CACHE_TYPE_MEMCACHE,
+#endif
 #ifdef USE_LIBHIREDIS
 			OIDC_CACHE_TYPE_REDIS,
 #endif
@@ -286,8 +288,10 @@ const char *oidc_parse_cache_type(apr_pool_t *pool, const char *arg,
 
 	if (apr_strnatcmp(arg, OIDC_CACHE_TYPE_SHM) == 0) {
 		*type = &oidc_cache_shm;
+#ifdef USE_MEMCACHE
 	} else if (apr_strnatcmp(arg, OIDC_CACHE_TYPE_MEMCACHE) == 0) {
 		*type = &oidc_cache_memcache;
+#endif
 	} else if (apr_strnatcmp(arg, OIDC_CACHE_TYPE_FILE) == 0) {
 		*type = &oidc_cache_file;
 #ifdef USE_LIBHIREDIS
@@ -376,7 +380,6 @@ const char *oidc_parse_boolean(apr_pool_t *pool, const char *arg,
 }
 
 #define OIDC_ENDPOINT_AUTH_CLIENT_SECRET_POST  "client_secret_post"
-#define OIDC_ENDPOINT_AUTH_CLIENT_SECRET_BASIC "client_secret_basic"
 #define OIDC_ENDPOINT_AUTH_CLIENT_SECRET_JWT   "client_secret_jwt"
 #define OIDC_ENDPOINT_AUTH_PRIVATE_KEY_JWT     "private_key_jwt"
 #define OIDC_ENDPOINT_AUTH_BEARER_ACCESS_TOKEN "bearer_access_token"
@@ -779,6 +782,34 @@ const char *oidc_parse_pass_userinfo_as(apr_pool_t *pool, const char *v1,
 	return NULL;
 }
 
+#define OIDC_LOGOUT_ON_ERROR_REFRESH_STR "logout_on_error"
+
+/*
+ * convert a "logout_on_error" to an integer
+ */
+static int oidc_parse_logout_on_error_refresh_as_str2int(const char *v) {
+	if (apr_strnatcmp(v, OIDC_LOGOUT_ON_ERROR_REFRESH_STR) == 0)
+		return OIDC_LOGOUT_ON_ERROR_REFRESH;
+	return OIDC_CONFIG_POS_INT_UNSET;
+}
+
+/*
+ * parse a "logout_on_error" value from the provided strings
+ */
+const char *oidc_parse_logout_on_error_refresh_as(apr_pool_t *pool, const char *v1,
+		int *int_value) {
+	static char *options[] = {
+			OIDC_LOGOUT_ON_ERROR_REFRESH_STR,
+			NULL };
+	const char *rv = NULL;
+	rv = oidc_valid_string_option(pool, v1, options);
+	if (rv != NULL)
+		return rv;
+	*int_value = oidc_parse_logout_on_error_refresh_as_str2int(v1);
+
+	return NULL;
+}
+
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_HEADER_STR "header"
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_POST_STR   "post"
 #define OIDC_OAUTH_ACCEPT_TOKEN_IN_QUERY_STR  "query"
@@ -955,6 +986,7 @@ const char *oidc_parse_set_claims_as(apr_pool_t *pool, const char *arg,
 #define OIDC_UNAUTH_ACTION_AUTH_STR "auth"
 #define OIDC_UNAUTH_ACTION_PASS_STR "pass"
 #define OIDC_UNAUTH_ACTION_401_STR  "401"
+#define OIDC_UNAUTH_ACTION_407_STR  "407"
 #define OIDC_UNAUTH_ACTION_410_STR  "410"
 
 /*
@@ -966,6 +998,7 @@ const char *oidc_parse_unauth_action(apr_pool_t *pool, const char *arg,
 			OIDC_UNAUTH_ACTION_AUTH_STR,
 			OIDC_UNAUTH_ACTION_PASS_STR,
 			OIDC_UNAUTH_ACTION_401_STR,
+			OIDC_UNAUTH_ACTION_407_STR,
 			OIDC_UNAUTH_ACTION_410_STR,
 			NULL };
 	const char *rv = oidc_valid_string_option(pool, arg, options);
@@ -978,6 +1011,8 @@ const char *oidc_parse_unauth_action(apr_pool_t *pool, const char *arg,
 		*action = OIDC_UNAUTH_PASS;
 	else if (apr_strnatcmp(arg, OIDC_UNAUTH_ACTION_401_STR) == 0)
 		*action = OIDC_UNAUTH_RETURN401;
+	else if (apr_strnatcmp(arg, OIDC_UNAUTH_ACTION_407_STR) == 0)
+		*action = OIDC_UNAUTH_RETURN407;
 	else if (apr_strnatcmp(arg, OIDC_UNAUTH_ACTION_410_STR) == 0)
 		*action = OIDC_UNAUTH_RETURN410;
 
@@ -1013,13 +1048,14 @@ const char *oidc_parse_unautz_action(apr_pool_t *pool, const char *arg,
 }
 
 /*
- * check if there's one valid entry in a string of arrays
+ * check if there's a valid entry in a string of arrays, with a preference
  */
 const char *oidc_valid_string_in_array(apr_pool_t *pool, json_t *json,
 		const char *key, oidc_valid_function_t valid_function, char **value,
-		apr_byte_t optional) {
+		apr_byte_t optional, const char *preference) {
 	int i = 0;
 	json_t *json_arr = json_object_get(json, key);
+	apr_byte_t found = FALSE;
 	if ((json_arr != NULL) && (json_is_array(json_arr))) {
 		for (i = 0; i < json_array_size(json_arr); i++) {
 			json_t *elem = json_array_get(json_arr, i);
@@ -1030,12 +1066,21 @@ const char *oidc_valid_string_in_array(apr_pool_t *pool, json_t *json,
 				continue;
 			}
 			if (valid_function(pool, json_string_value(elem)) == NULL) {
-				if (value != NULL)
-					*value = apr_pstrdup(pool, json_string_value(elem));
-				break;
+				found = TRUE;
+				if (value != NULL) {
+					if ((preference != NULL)
+							&& (apr_strnatcmp(json_string_value(elem),
+									preference) == 0)) {
+						*value = apr_pstrdup(pool, json_string_value(elem));
+						break;
+					}
+					if (*value == NULL) {
+						*value = apr_pstrdup(pool, json_string_value(elem));
+					}
+				}
 			}
 		}
-		if (i == json_array_size(json_arr)) {
+		if (found == FALSE) {
 			return apr_psprintf(pool,
 					"could not find a valid array string element for entry \"%s\"",
 					key);
