@@ -502,9 +502,12 @@ char *oidc_get_current_url(request_rec *r) {
 	if ((path) && (path[0] != '/')) {
 		memset(&uri, 0, sizeof(apr_uri_t));
 		if (apr_uri_parse(r->pool, r->uri, &uri) == APR_SUCCESS)
-			path = apr_pstrcat(r->pool, uri.path, (r->args != NULL && *r->args != '\0' ? "?" : ""), r->args, NULL);
+			path = apr_pstrcat(r->pool, uri.path,
+					(r->args != NULL && *r->args != '\0' ? "?" : ""), r->args,
+					NULL);
 		else
-			oidc_warn(r, "apr_uri_parse failed on non-relative URL: %s", r->uri);
+			oidc_warn(r, "apr_uri_parse failed on non-relative URL: %s",
+					r->uri);
 	} else {
 		/* make sure we retain URL-encoded characters original URL that we send the user back to */
 		path = r->unparsed_uri;
@@ -649,7 +652,7 @@ char *oidc_util_http_query_encoded_url(request_rec *r, const char *url,
 /*
  * construct form-encoded POST data
  */
-static char *oidc_util_http_form_encoded_data(request_rec *r,
+char *oidc_util_http_form_encoded_data(request_rec *r,
 		const apr_table_t *params) {
 	char *data = NULL;
 	if ((params != NULL) && (apr_table_elts(params)->nelts > 0)) {
@@ -928,6 +931,27 @@ static char *oidc_util_get_cookie_path(request_rec *r) {
 
 #define OIDC_COOKIE_MAX_SIZE            4093
 
+#define OIDC_SET_COOKIE_APPEND_ENV_VAR  "OIDC_SET_COOKIE_APPEND"
+
+const char *oidc_util_set_cookie_append_value(request_rec *r, oidc_cfg *c) {
+	const char *env_var_value = NULL;
+
+	if (r->subprocess_env != NULL)
+		env_var_value = apr_table_get(r->subprocess_env,
+				OIDC_SET_COOKIE_APPEND_ENV_VAR);
+
+	if (env_var_value == NULL) {
+		oidc_debug(r, "no cookie append environment variable %s found",
+				OIDC_SET_COOKIE_APPEND_ENV_VAR);
+		return NULL;
+	}
+
+	oidc_debug(r, "cookie append environment variable %s=%s found",
+			OIDC_SET_COOKIE_APPEND_ENV_VAR, env_var_value);
+
+	return env_var_value;
+}
+
 /*
  * set a cookie in the HTTP response headers
  */
@@ -937,6 +961,7 @@ void oidc_util_set_cookie(request_rec *r, const char *cookieName,
 	oidc_cfg *c = ap_get_module_config(r->server->module_config,
 			&auth_openidc_module);
 	char *headerString, *expiresString = NULL;
+	const char *appendString = NULL;
 
 	/* see if we need to clear the cookie */
 	if (apr_strnatcmp(cookieValue, "") == 0)
@@ -972,7 +997,11 @@ void oidc_util_set_cookie(request_rec *r, const char *cookieName,
 		headerString = apr_psprintf(r->pool, "%s; %s", headerString,
 				OIDC_COOKIE_FLAG_HTTP_ONLY);
 
-	if (ext != NULL)
+	appendString = oidc_util_set_cookie_append_value(r, c);
+	if (appendString != NULL)
+		headerString = apr_psprintf(r->pool, "%s; %s", headerString,
+				appendString);
+	else if (ext != NULL)
 		headerString = apr_psprintf(r->pool, "%s; %s", headerString, ext);
 
 	/* sanity check on overall cookie value size */
@@ -1093,6 +1122,23 @@ char *oidc_util_get_chunked_cookie(request_rec *r, const char *cookieName,
 	return cookieValue;
 }
 
+/*
+ * unset all chunked cookies, including the counter cookie, if they exist
+ */
+static void oidc_util_clear_chunked_cookie(request_rec *r,
+		const char *cookieName, apr_time_t expires, const char *ext) {
+	int i = 0;
+	int chunkCount = oidc_util_get_chunked_count(r, cookieName);
+	if (chunkCount > 0) {
+		for (i = 0; i < chunkCount; i++)
+			oidc_util_set_cookie(r,
+					oidc_util_get_chunk_cookie_name(r, cookieName, i), "",
+					expires, ext);
+		oidc_util_set_cookie(r, oidc_util_get_chunk_count_name(r, cookieName),
+				"", expires, ext);
+	}
+}
+
 /*
  * set a cookie value that is split over a number of chunked cookies
  */
@@ -1101,28 +1147,20 @@ void oidc_util_set_chunked_cookie(request_rec *r, const char *cookieName,
 		const char *ext) {
 	int i = 0;
 	int cookieLength = strlen(cookieValue);
-	char *chunkCountName = oidc_util_get_chunk_count_name(r, cookieName);
 	char *chunkValue = NULL;
 
 	/* see if we need to chunk at all */
 	if ((chunkSize == 0)
 			|| ((cookieLength > 0) && (cookieLength < chunkSize))) {
 		oidc_util_set_cookie(r, cookieName, cookieValue, expires, ext);
+		oidc_util_clear_chunked_cookie(r, cookieName, expires, ext);
 		return;
 	}
 
 	/* see if we need to clear a possibly chunked cookie */
 	if (cookieLength == 0) {
-		int chunkCount = oidc_util_get_chunked_count(r, cookieName);
-		if (chunkCount > 0) {
-			for (i = 0; i < chunkCount; i++)
-				oidc_util_set_cookie(r,
-						oidc_util_get_chunk_cookie_name(r, cookieName, i), "",
-						expires, ext);
-			oidc_util_set_cookie(r, chunkCountName, "", expires, ext);
-		} else {
-			oidc_util_set_cookie(r, cookieName, "", expires, ext);
-		}
+		oidc_util_set_cookie(r, cookieName, "", expires, ext);
+		oidc_util_clear_chunked_cookie(r, cookieName, expires, ext);
 		return;
 	}
 
@@ -1136,8 +1174,9 @@ void oidc_util_set_chunked_cookie(request_rec *r, const char *cookieName,
 				oidc_util_get_chunk_cookie_name(r, cookieName, i), chunkValue,
 				expires, ext);
 	};
-	oidc_util_set_cookie(r, chunkCountName,
+	oidc_util_set_cookie(r, oidc_util_get_chunk_count_name(r, cookieName),
 			apr_psprintf(r->pool, "%d", chunkCountValue), expires, ext);
+	oidc_util_set_cookie(r, cookieName, "", expires, ext);
 }
 
 /*
@@ -1332,6 +1371,15 @@ int oidc_util_http_send(request_rec *r, const char *data, size_t data_len,
 		return HTTP_INTERNAL_SERVER_ERROR;
 	}
 	//r->status = success_rvalue;
+
+	if ((success_rvalue == OK) && (r->user == NULL)) {
+		/*
+		 * satisfy Apache 2.4 mod_authz_core:
+		 * prevent it to return HTTP 500 after sending content
+		 */
+		r->user = "";
+	}
+
 	return success_rvalue;
 }
 
@@ -1488,19 +1536,55 @@ apr_byte_t oidc_util_read_form_encoded_params(request_rec *r,
 	return TRUE;
 }
 
+static void oidc_userdata_set_post_param(request_rec *r,
+		const char *post_param_name, const char *post_param_value) {
+	apr_table_t *userdata_post_params = NULL;
+	apr_pool_userdata_get((void **) &userdata_post_params,
+			OIDC_USERDATA_POST_PARAMS_KEY, r->pool);
+	if (userdata_post_params == NULL)
+		userdata_post_params = apr_table_make(r->pool, 1);
+	apr_table_set(userdata_post_params, post_param_name, post_param_value);
+	apr_pool_userdata_set(userdata_post_params, OIDC_USERDATA_POST_PARAMS_KEY,
+			NULL, r->pool);
+
+}
+
 /*
  * read the POST parameters in to a table
  */
-apr_byte_t oidc_util_read_post_params(request_rec *r, apr_table_t *table) {
+apr_byte_t oidc_util_read_post_params(request_rec *r, apr_table_t *table,
+		apr_byte_t propagate, const char *strip_param_name) {
+	apr_byte_t rc = FALSE;
 	char *data = NULL;
+	const apr_array_header_t *arr = NULL;
+	const apr_table_entry_t *elts = NULL;
+	int i = 0;
+	const char *content_type = NULL;
 
-	if (r->method_number != M_POST)
-		return FALSE;
+	content_type = oidc_util_hdr_in_content_type_get(r);
+	if ((r->method_number != M_POST) || (apr_strnatcmp(content_type,
+			OIDC_CONTENT_TYPE_FORM_ENCODED) != 0))
+		goto end;
 
 	if (oidc_util_read(r, &data) != TRUE)
-		return FALSE;
+		goto end;
 
-	return oidc_util_read_form_encoded_params(r, table, data);
+	rc = oidc_util_read_form_encoded_params(r, table, data);
+	if (rc != TRUE)
+		goto end;
+
+	if (propagate == FALSE)
+		goto end;
+
+	arr = apr_table_elts(table);
+	elts = (const apr_table_entry_t*) arr->elts;
+	for (i = 0; i < arr->nelts; i++)
+		if (apr_strnatcmp(elts[i].key, strip_param_name) != 0)
+			oidc_userdata_set_post_param(r, elts[i].key, elts[i].val);
+
+	end:
+
+	return rc;
 }
 
 /*
@@ -1690,8 +1774,6 @@ void oidc_util_set_app_info(request_rec *r, const char *s_key,
 		const char *s_value, const char *claim_prefix, apr_byte_t as_header,
 		apr_byte_t as_env_var) {
 
-	apr_table_t *env = NULL;
-
 	/* construct the header name, cq. put the prefix in front of a normalized key name */
 	const char *s_name = apr_psprintf(r->pool, "%s%s", claim_prefix,
 			oidc_normalize_header_name(r, s_key));
@@ -1705,11 +1787,7 @@ void oidc_util_set_app_info(request_rec *r, const char *s_key,
 		oidc_debug(r, "setting environment variable \"%s: %s\"", s_name,
 				s_value);
 
-		apr_pool_userdata_get((void **) &env, OIDC_USERDATA_ENV_KEY, r->pool);
-		if (env == NULL)
-			env = apr_table_make(r->pool, 10);
-		apr_table_set(env, s_name, s_value);
-		apr_pool_userdata_set(env, OIDC_USERDATA_ENV_KEY, NULL, r->pool);
+		apr_table_set(r->subprocess_env, s_name, s_value);
 	}
 }
 
@@ -2336,6 +2414,10 @@ const char *oidc_util_hdr_in_content_type_get(const request_rec *r) {
 	return oidc_util_hdr_in_get(r, OIDC_HTTP_HDR_CONTENT_TYPE);
 }
 
+const char *oidc_util_hdr_in_content_length_get(const request_rec *r) {
+	return oidc_util_hdr_in_get(r, OIDC_HTTP_HDR_CONTENT_LENGTH);
+}
+
 const char *oidc_util_hdr_in_x_requested_with_get(const request_rec *r) {
 	return oidc_util_hdr_in_get(r, OIDC_HTTP_HDR_X_REQUESTED_WITH);
 }
@@ -2552,4 +2634,3 @@ out_err:
 	// TODO: we don't know which token binding the client supports, do we ?
 	return FALSE;
 }
-
