@@ -2,6 +2,7 @@
 #include <http_core.h>
 #include <http_config.h>
 #include <apr_global_mutex.h>
+#include <apr_strings.h>
 #include <apr_lib.h>
 #include <http_log.h>
 
@@ -36,8 +37,26 @@ AP_DECLARE(long) ap_get_client_block(request_rec * r, char * buffer,
 	return 0;
 }
 
-AP_DECLARE(char *) ap_getword(apr_pool_t *p, const char **line, char stop) {
-	return "";
+AP_DECLARE(char *) ap_getword(apr_pool_t *atrans, const char **line, char stop) {
+	const char *pos = *line;
+	int len;
+	char *res;
+
+	while ((*pos != stop) && *pos) {
+		++pos;
+	}
+
+	len = pos - *line;
+	res = apr_pstrmemdup(atrans, *line, len);
+
+	if (stop) {
+		while (*pos == stop) {
+			++pos;
+		}
+	}
+	*line = pos;
+
+	return res;
 }
 
 static char *substring_conf(apr_pool_t *p, const char *start, int len,
@@ -125,6 +144,10 @@ AP_DECLARE(int) ap_hook_fixups(request_rec *r) {
 	return 0;
 }
 
+AP_DECLARE(int) ap_hook_insert_filter(request_rec *r) {
+	return 0;
+}
+
 AP_DECLARE(void) ap_hook_post_config(
 		int (*post_config)(apr_pool_t *pool, apr_pool_t *p1, apr_pool_t *p2,
 				server_rec *s), const char * const *aszPre,
@@ -244,3 +267,21 @@ AP_DECLARE(void) ap_log_error_(const char *file, int line, int module_index,
 		AP_DECLARE(char *) ap_server_root_relative(apr_pool_t *p, const char *file) {
 			return "";
 		}
+
+		AP_DECLARE(ap_filter_t *) ap_add_input_filter(const char *name, void *ctx,
+				request_rec *r, conn_rec *c) {
+			return NULL;
+		}
+
+		AP_DECLARE(apr_status_t) ap_get_brigade(ap_filter_t *filter,
+				apr_bucket_brigade *bucket, ap_input_mode_t mode, apr_read_type_e block,
+				apr_off_t readbytes) {
+			return APR_SUCCESS;
+		}
+
+		AP_DECLARE(ap_filter_rec_t *) ap_register_input_filter(const char *name,
+				ap_in_filter_func filter_func,
+				ap_init_filter_func filter_init,
+				ap_filter_type ftype) {
+			return NULL;
+		}
